@@ -1,19 +1,17 @@
 """Base classes and function for REST, WMS, and WMF services."""
 import itertools
 import uuid
-from pathlib import Path
-from types import SimpleNamespace
-from typing import Any, Dict, Iterator, List, Optional, Tuple, Union
+from ssl import SSLContext
+from typing import Any, Dict, Iterator, List, NamedTuple, Optional, Sequence, Tuple, Union
 
 import async_retriever as ar
 import cytoolz as tlz
 import pyproj
 import shapely.ops as ops
-import yaml
 from shapely.geometry import LineString, MultiPoint, MultiPolygon, Point, Polygon
 
 from . import utils
-from .core import DEF_CRS, EXPIRE, ArcGISRESTfulBase, WFSBase, WMSBase
+from .core import DEF_CRS, ArcGISRESTfulBase, WFSBase, WMSBase
 from .exceptions import InvalidInputType, InvalidInputValue, ZeroMatched
 
 
@@ -57,10 +55,6 @@ class ArcGISRESTful:
         If ``True`` in case there are any failed queries, no retrying attempts
         is done and object IDs of the failed requests is saved to a text file
         which its path can be accessed via ``self.client.failed_path``.
-    expire_after : int, optional
-        Expiration time for response caching in seconds, defaults to -1 (never expire).
-    disable_caching : bool, optional
-        If ``True``, disable caching requests, defaults to False.
     """
 
     def __init__(
@@ -73,8 +67,6 @@ class ArcGISRESTful:
         max_workers: int = 1,
         verbose: bool = False,
         disable_retry: bool = False,
-        expire_after: float = EXPIRE,
-        disable_caching: bool = False,
     ) -> None:
         self.client = ArcGISRESTfulBase(
             base_url=base_url,
@@ -85,8 +77,6 @@ class ArcGISRESTful:
             max_workers=max_workers,
             verbose=verbose,
             disable_retry=disable_retry,
-            expire_after=expire_after,
-            disable_caching=disable_caching,
         )
 
     def oids_bygeom(
@@ -179,8 +169,9 @@ class ArcGISRESTful:
         resp = self.client.get_response(self.client.query_url, [payload], method="POST")[0]
         try:
             return self.partition_oids(resp["objectIds"])
-        except KeyError as ex:
-            raise ZeroMatched(resp["error"]["message"]) from ex
+        except (KeyError, TypeError) as ex:
+            msg = resp["error"]["message"] if "error" in resp else "No matched records"
+            raise ZeroMatched(msg) from ex
 
     def oids_byfield(self, field: str, ids: Union[str, List[str]]) -> Iterator[Tuple[str, ...]]:
         """Get Object IDs based on a list of field IDs.
@@ -200,11 +191,16 @@ class ArcGISRESTful:
         if field not in self.client.valid_fields:
             raise InvalidInputValue("field", self.client.valid_fields)
 
+        if not isinstance(ids, Sequence):
+            raise InvalidInputType("ids", "str or list")
+
+        ids_ls = [ids] if isinstance(ids, str) else ids
+
         ftype = self.client.field_types[field]
         if "string" in ftype:
-            fids = ", ".join(f"'{i}'" for i in ids)
+            fids = ", ".join(f"'{i}'" for i in ids_ls)
         else:
-            fids = ", ".join(f"{i}" for i in ids)
+            fids = ", ".join(f"{i}" for i in ids_ls)
 
         return self.oids_bysql(f"{field} IN ({fids})")
 
@@ -237,11 +233,12 @@ class ArcGISRESTful:
         }
         self.client.request_id = uuid.uuid4().hex
 
-        resp = self.client.get_response(self.client.query_url, [payload])[0]
+        resp = self.client.get_response(self.client.query_url, [payload], method="POST")[0]
         try:
             return self.partition_oids(resp["objectIds"])
-        except KeyError as ex:
-            raise ZeroMatched(resp["error"]["message"]) from ex
+        except (KeyError, TypeError) as ex:
+            msg = resp["error"]["message"] if "error" in resp else "No matched records"
+            raise ZeroMatched(msg) from ex
 
     def partition_oids(self, oids: Union[List[int], int]) -> Iterator[Tuple[str, ...]]:
         """Partition feature IDs based on ``self.max_nrecords``.
@@ -288,7 +285,7 @@ class ArcGISRESTful:
         return self.client.__repr__()
 
 
-class WMS(WMSBase):
+class WMS:
     """Get data from a WMS service within a geometry or bounding box.
 
     Parameters
@@ -310,10 +307,9 @@ class WMS(WMSBase):
         Validate the input arguments from the WMS service, defaults to True. Set this
         to False if you are sure all the WMS settings such as layer and crs are correct
         to avoid sending extra requests.
-    expire_after : int, optional
-        Expiration time for response caching in seconds, defaults to -1 (never expire).
-    disable_caching : bool, optional
-        If ``True``, disable caching requests, defaults to False.
+    ssl : bool or SSLContext, optional
+        SSLContext to use for the connection, defaults to None. Set to False to disable
+        SSL certification verification.
     """
 
     def __init__(
@@ -324,22 +320,29 @@ class WMS(WMSBase):
         version: str = "1.3.0",
         crs: str = DEF_CRS,
         validation: bool = True,
-        expire_after: float = EXPIRE,
-        disable_caching: bool = False,
+        ssl: Union[SSLContext, bool, None] = None,
     ) -> None:
-        super().__init__(
+        self.client = WMSBase(
             url=url,
             layers=layers,
             outformat=outformat,
             version=version,
             crs=crs,
-            expire_after=expire_after,
-            disable_caching=disable_caching,
         )
-
-        self.layers = [self.layers] if isinstance(self.layers, str) else self.layers
+        self.url = self.client.url
+        self.outformat = self.client.outformat
+        self.version = self.client.version
+        self.crs = self.client.crs
+        self.ssl = ssl
+        self.layers = (
+            [self.client.layers] if isinstance(self.client.layers, str) else self.client.layers
+        )
         if validation:
-            self.validate_wms()
+            self.client.validate_wms()
+
+    def get_validlayers(self) -> Dict[str, str]:
+        """Get the layers supported by the WMS service."""
+        return self.client.get_validlayers()
 
     def getmap_bybox(
         self,
@@ -421,11 +424,14 @@ class WMS(WMSBase):
             [self.url] * len(payloads),
             [{"params": p} for p in payloads],
             max_workers=4,
-            expire_after=self.expire_after,
-            disable=self.disable_caching,
+            ssl=self.ssl,
         )
         return dict(zip(layers, rbinary))
 
+    def __repr__(self) -> str:
+        """Print the services properties."""
+        return self.client.__repr__()
+
 
 class WFS(WFSBase):
     """Data from any WFS service within a geometry or by featureid.
@@ -458,10 +464,6 @@ class WFS(WFSBase):
         Validate the input arguments from the WFS service, defaults to True. Set this
         to False if you are sure all the WFS settings such as layer and crs are correct
         to avoid sending extra requests.
-    expire_after : int, optional
-        Expiration time for response caching in seconds, defaults to -1 (never expire).
-    disable_caching : bool, optional
-        If ``True``, disable caching requests, defaults to False.
     """
 
     def __init__(
@@ -474,8 +476,6 @@ class WFS(WFSBase):
         read_method: str = "json",
         max_nrecords: int = 1000,
         validation: bool = True,
-        expire_after: float = EXPIRE,
-        disable_caching: bool = False,
     ) -> None:
         super().__init__(
             url=url,
@@ -485,8 +485,6 @@ class WFS(WFSBase):
             crs=crs,
             read_method=read_method,
             max_nrecords=max_nrecords,
-            expire_after=expire_after,
-            disable_caching=disable_caching,
         )
 
         if validation:
@@ -534,13 +532,7 @@ class WFS(WFSBase):
             "srsName": self.crs,
         }
 
-        return ar.retrieve(
-            [self.url],
-            self.read_method,
-            [{"params": payload}],
-            expire_after=self.expire_after,
-            disable=self.disable_caching,
-        )[0]
+        return ar.retrieve([self.url], self.read_method, [{"params": payload}])[0]
 
     def getfeature_bygeom(
         self,
@@ -688,52 +680,77 @@ class WFS(WFSBase):
         }
 
         if method == "GET":
-            return ar.retrieve(
-                [self.url],
-                self.read_method,
-                [{"params": payload}],
-                expire_after=self.expire_after,
-                disable=self.disable_caching,
-            )[0]
+            return ar.retrieve([self.url], self.read_method, [{"params": payload}])[0]
 
         headers = {"content-type": "application/x-www-form-urlencoded"}
         return ar.retrieve(
-            [self.url],
-            self.read_method,
-            [{"data": payload, "headers": headers}],
-            "POST",
-            expire_after=self.expire_after,
-            disable=self.disable_caching,
+            [self.url], self.read_method, [{"data": payload, "headers": headers}], "POST"
         )[0]
 
 
-class ServiceURL:
-    """Base URLs of the supported services."""
+class HttpURLs(NamedTuple):
+    """URLs of the supported HTTP services."""
 
-    def __init__(self) -> None:
-        fpath = Path(__file__).parent.joinpath("static/urls.yml")
-        with open(fpath) as fp:
-            self.urls = yaml.safe_load(fp)
+    ssebopeta: str = "https://edcintl.cr.usgs.gov/downloads/sciweb1/shared/uswem/web/conus/eta/modis_eta/daily/downloads"
 
-    def _make_nt(self, service: str) -> SimpleNamespace:
-        return SimpleNamespace(**self.urls[service])
 
-    @property
-    def restful(self) -> SimpleNamespace:
-        """Read RESTful URLs from the source yml file."""
-        return self._make_nt("restful")
+class RESTfulURLs(NamedTuple):
+    """URLs of the supported RESTful services."""
 
-    @property
-    def wms(self) -> SimpleNamespace:
-        """Read WMS URLs from the source yml file."""
-        return self._make_nt("wms")
+    daymet: str = "https://thredds.daac.ornl.gov/thredds/ncss/ornldaac"
+    daymet_point: str = "https://daymet.ornl.gov/single-pixel/api/data"
+    fema: str = "https://hazards.fema.gov/gis/nfhl/rest/services/public/NFHL/MapServer"
+    fws: str = "https://www.fws.gov/wetlandsmapservice/rest/services"
+    nldi: str = "https://labs.waterdata.usgs.gov/api/nldi"
+    nwis: str = "https://waterservices.usgs.gov/nwis"
+    wbd: str = "https://hydro.nationalmap.gov/arcgis/rest/services/wbd/MapServer"
+    nhd: str = "https://hydro.nationalmap.gov/arcgis/rest/services/nhd/MapServer"
+    nhdplushr: str = "https://hydro.nationalmap.gov/arcgis/rest/services/NHDPlus_HR/MapServer"
+    nhdhr_edits: str = "https://edits.nationalmap.gov/arcgis/rest/services/HEM/NHDHigh/MapServer"
+    nhdplushr_edits: str = (
+        "https://edits.nationalmap.gov/arcgis/rest/services/NHDPlus_HR/NHDPlus_HR/MapServer"
+    )
+    nhdplus_epa: str = "https://watersgeo.epa.gov/arcgis/rest/services/NHDPlus/NHDPlus/MapServer"
+    nhd_fabric: str = (
+        "https://watersgeo.epa.gov/arcgis/rest/services/Support/CatchmentFabric/MapServer"
+    )
+    nid: str = "https://nid.sec.usace.army.mil/api"
+    airmap: str = "https://api.airmap.com/elevation/v1/ele"
+    nm_pqs: str = "https://nationalmap.gov/epqs/pqs.php"
+    pygeoapi: str = "https://labs.waterdata.usgs.gov/api/nldi/pygeoapi/processes"
+    nm_3dep_index: str = (
+        "https://index.nationalmap.gov/arcgis/rest/services/3DEPElevationIndex/MapServer"
+    )
+    geoconnex: str = "https://reference.geoconnex.us"
 
-    @property
-    def wfs(self) -> SimpleNamespace:
-        """Read WFS URLs from the source yml file."""
-        return self._make_nt("wfs")
 
-    @property
-    def http(self) -> SimpleNamespace:
-        """Read HTTP URLs from the source yml file."""
-        return self._make_nt("http")
+class WFSURLs(NamedTuple):
+    """URLs of the supported WFS services."""
+
+    fema: str = "https://hazards.fema.gov/gis/nfhl/services/public/NFHL/MapServer/WFSServer"
+    waterdata: str = "https://labs.waterdata.usgs.gov/geoserver/wmadata/ows"
+
+
+class WMSURLs(NamedTuple):
+    """URLs of the supported WMS services."""
+
+    fema: str = "https://hazards.fema.gov/gis/nfhl/rest/services/public/NFHLWMS/MapServer/WMSServer"
+    fws: str = (
+        "https://www.fws.gov/wetlandsmapservice/services/Wetlands_Raster/ImageServer/WMSServer"
+    )
+    mrlc: str = "https://www.mrlc.gov/geoserver/mrlc_download/wms"
+    nm_3dep: str = (
+        "https://elevation.nationalmap.gov/arcgis/services/3DEPElevation/ImageServer/WMSServer"
+    )
+    gebco: str = (
+        "https://www.gebco.net/data_and_products/gebco_web_services/web_map_service/mapserv"
+    )
+
+
+class ServiceURL(NamedTuple):
+    """URLs of the supported services."""
+
+    http: HttpURLs = HttpURLs()
+    restful: RESTfulURLs = RESTfulURLs()
+    wfs: WFSURLs = WFSURLs()
+    wms: WMSURLs = WMSURLs()
