@@ -155,7 +155,7 @@ class TestWMS:
     def test_valid_crs(self):
         """Get WMS valid CRSs"""
         crs = utils.valid_wms_crs(self.wms_url)
-        assert sorted(crs) == ['epsg:3395', 'epsg:3857', 'epsg:4326']
+        assert sorted(crs) == ["epsg:3395", "epsg:3857", "epsg:4326"]
 
 
 class TestWFS:
@@ -254,7 +254,7 @@ def test_esriquery():
 
 def test_urls():
     """Service URLs"""
-    assert len(ServiceURL().__dict__["urls"]) == 4
+    assert len(ServiceURL()) == 4
 
 
 def test_show_versions():
